// main/iot/things/bilibili_fans.cc
#include "iot/thing.h"
#include "board.h"

#include <esp_log.h>
#include <cJSON.h>
#include "esp_event.h"
#include "esp_wifi.h"
#define TAG "BilibiliFans"

namespace iot
{

    class BilibiliFans : public Thing
    {
    private:
        std::string uid_ = "你的B站UID";  、//396355825
        int follower_count_ = -1;
        bool refresh_in_progress_ = false;
        esp_timer_handle_t clock_timer_handle_ = nullptr;

        // int follower_count_ = 0;

        bool wifi_connected_ = false; // 标记 Wi-Fi 状态

        bool RefreshFollowerCount()
        {
            if (uid_.empty())
            {
                ESP_LOGE(TAG, "UID is not set");
                return false;
            }

            if (refresh_in_progress_)
            {
                ESP_LOGI(TAG, "Refresh already in progress");
                return false;
            }

            refresh_in_progress_ = true;

            auto &board = Board::GetInstance();
            auto http = board.CreateHttp();

            std::string url = "https://api.bilibili.com/x/relation/stat?vmid=" + uid_;
            http->SetHeader("User-Agent", "Mozilla/5.0");

            if (!http->Open("GET", url))
            {
                ESP_LOGE(TAG, "Failed to open HTTP connection");
                delete http;
                refresh_in_progress_ = false;
                return false;
            }

            std::string response = http->GetBody();
            delete http;

            cJSON *root = cJSON_Parse(response.c_str());
            if (root == NULL)
            {
                ESP_LOGE(TAG, "Failed to parse JSON response");
                refresh_in_progress_ = false;
                return false;
            }

            cJSON *code = cJSON_GetObjectItem(root, "code");
            if (code == NULL || code->valueint != 0)
            {
                ESP_LOGE(TAG, "API returned error code");
                cJSON_Delete(root);
                refresh_in_progress_ = false;
                return false;
            }

            cJSON *data = cJSON_GetObjectItem(root, "data");
            if (data == NULL)
            {
                ESP_LOGE(TAG, "No data field in response");
                cJSON_Delete(root);
                refresh_in_progress_ = false;
                return false;
            }

            cJSON *follower = cJSON_GetObjectItem(data, "follower");
            if (follower == NULL)
            {
                ESP_LOGE(TAG, "No follower field in data");
                cJSON_Delete(root);
                refresh_in_progress_ = false;
                return false;
            }

            follower_count_ = follower->valueint;
            ESP_LOGI(TAG, "B站用户 %s 的粉丝数：%d", uid_.c_str(), follower_count_);

            cJSON_Delete(root);
            refresh_in_progress_ = false;
            return true;
        }

        static void wifi_event_handler(void *arg, esp_event_base_t event_base, int32_t event_id, void *event_data)
        {
            BilibiliFans *instance = static_cast<BilibiliFans *>(arg);

            if (event_base == IP_EVENT && event_id == IP_EVENT_STA_GOT_IP)
            {
                ESP_LOGI("BiliFans", "Wi-Fi connected, updating fans count...");
                instance->OnWifiConnected();
            }
            if (event_base == WIFI_EVENT && event_id == WIFI_EVENT_STA_DISCONNECTED)
            {
                ESP_LOGW("BiliFans", "Wi-Fi disconnected, pausing update timer...");
                instance->OnWifiDisconnected();
            }
        }

        void OnWifiConnected()
        {
            wifi_connected_ = true;
            UpdateFansCount();
            if (clock_timer_handle_)
            {
                esp_timer_start_periodic(clock_timer_handle_, 1 * 60 * 1000000);
            }
        }

        void OnWifiDisconnected()
        {
            wifi_connected_ = false;
            if (clock_timer_handle_)
            {
                esp_timer_stop(clock_timer_handle_);
            }
        }

        void UpdateFansCount()
        {
            RefreshFollowerCount();
            if (follower_count_ < 0)
            {
                ESP_LOGE(TAG, "Failed to get fans count");
            }
            ESP_LOGI(TAG, "Fans count updated: %d", follower_count_);
        }
        void InitializeEvent()
        {
            esp_timer_create_args_t clock_timer_args = {
                .callback = [](void *arg)
                {
                    BilibiliFans *instance = (BilibiliFans *)(arg);
                    instance->UpdateFansCount();
                },
                .arg = this,
                .dispatch_method = ESP_TIMER_TASK,
                .name = "BiliFansUpdateTimer",
                .skip_unhandled_events = true};
            esp_timer_create(&clock_timer_args, &clock_timer_handle_);

            ESP_ERROR_CHECK(esp_event_handler_register(IP_EVENT, IP_EVENT_STA_GOT_IP, &BilibiliFans::wifi_event_handler, this));
            ESP_ERROR_CHECK(esp_event_handler_register(WIFI_EVENT, WIFI_EVENT_STA_DISCONNECTED, &BilibiliFans::wifi_event_handler, this));
        }


    public:
        BilibiliFans() : Thing("BilibiliFans", "B站粉丝数量")
        {
            // 定义属性
            InitializeEvent();
            properties_.AddStringProperty("uid", "B站用户ID", [this]() -> std::string
                                          { return uid_; });

            properties_.AddNumberProperty("follower_count", "粉丝数量", [this]() -> int
                                          { return follower_count_; });

            // 定义方法
            // methods_.AddMethod("SetUID", "设置B站用户ID", ParameterList({
            //     Parameter("uid", "B站用户ID", kValueTypeString, true)
            // }), [this](const ParameterList& parameters) {
            //     uid_ = parameters["uid"].string();
            //     ESP_LOGI(TAG, "设置B站用户ID为: %s", uid_.c_str());
            //     RefreshFollowerCount();
            // });

            methods_.AddMethod("Refresh", "刷新粉丝数量", ParameterList(), [this](const ParameterList &parameters)
                               {  
            ESP_LOGI(TAG, "刷新B站粉丝数量");  
            RefreshFollowerCount(); });

            methods_.AddMethod("Report", "汇报粉丝数量", ParameterList(), [this](const ParameterList &parameters)
                               {
                                   ESP_LOGI(TAG, "汇报B站粉丝数量");
                                //    RefreshFollowerCount();
                                   return follower_count_;
                                   // 注意：实际的汇报效果是通过LLM理解粉丝数属性值来实现的
                               });
        }

        ~BilibiliFans()
        {
            if (clock_timer_handle_)
            {
                esp_timer_stop(clock_timer_handle_);
                esp_timer_delete(clock_timer_handle_);
                clock_timer_handle_ = nullptr;
            }

            esp_event_handler_unregister(IP_EVENT, IP_EVENT_STA_GOT_IP, &BilibiliFans::wifi_event_handler);
            esp_event_handler_unregister(WIFI_EVENT, WIFI_EVENT_STA_DISCONNECTED, &BilibiliFans::wifi_event_handler);
        }
    };


} // namespace iot

DECLARE_THING(BilibiliFans);